/*
Module : MfcCDDB.H
Purpose: Defines the interface for a collection of MFC classes to wrap access to CDDB

Copyright (c) 1999 - 2008 by PJ Naughter.  (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


///////////////////////////////// Macros / Defines ////////////////////////////

#pragma once

#ifndef __MFCCDDB_H__
#define __MFCCDDB_H__

#ifndef __AFXTEMPL_H__
#include <afxtempl.h>
#pragma message("to avoid this message, you should put afxtempl.h in your pre compiled header (normally stdafx.h)")
#endif

#include "SocMFC.h" //If you get a compilation error about this missing header file, then you need to download my CWSocket class from http://www.naughter.com/w3mfc.html

#ifndef MFCCDDB_EXT_CLASS
#define MFCCDDB_EXT_CLASS
#endif


//////////////////////////// Classes //////////////////////////////////////////

class MFCCDDB_EXT_CLASS CCDDBException : public CException
{
public:
//Constructors / Destructors
  CCDDBException(HRESULT hr, const CString& sLastResponse = _T("")); 
	CCDDBException(DWORD dwError, DWORD dwFacility = FACILITY_WIN32, const CString& sLastResponse = _T(""));
	CCDDBException(BOOL bMMError, DWORD dwError, const CString& sLastResponse = _T(""));

//Methods
#ifdef _DEBUG
	virtual void Dump(CDumpContext& dc) const;
#endif
	virtual BOOL GetErrorMessage(__out_ecount_z(nMaxError) LPTSTR lpstrError, __in UINT nMaxError,	__out_opt PUINT pnHelpContext = NULL);
	CString GetErrorMessage();

//Data members
  BOOL m_bMMError;
	HRESULT m_hr;
  CString m_sLastResponse;

protected:
	DECLARE_DYNAMIC(CCDDBException)
};

//Class which encapsulates a CDDB site
class MFCCDDB_EXT_CLASS CCDDBSite
{
public:
//Constructors / Destructors
  CCDDBSite(BOOL bRetrieve = TRUE);
  CCDDBSite(const CCDDBSite& site);

//Methods
  CCDDBSite& operator=(const CCDDBSite& site);
  
//Data variables
  CString m_sSite;             //The site name e.g. "freedb.freedb.org"
  int     m_nPort;             //The port number to connect on, normally the HTTP port i.e. 80
  CString m_sAddress;          //The address of the HTTP file to use, normally either "/~cddb/cddb.cgi" or "/~cddb/submit.cgi"
  BOOL    m_bNorthing;         //Is this site north of the equator
  int     m_nLatitudeMinutes;  //Minutes of degrees north of the equator 
  BOOL    m_bEasting;          //Is this site east of the prime meridian
  int     m_nLongitudeMinutes; //Minutes of degrees east of the prime meridian
  CString m_sDescription;      //Short description of the geographical location of the site
  CString m_sProtocol;         //The protocol the server uses (Note this is only read and not actively used by MfcCDDB)

protected:
  friend class CCDDBQueryResult;
};

//Class which represents the single instance of the
//results of a call to CCDDB::Query
class MFCCDDB_EXT_CLASS CCDDBQueryResult
{
public:
  CCDDBQueryResult();
  CCDDBQueryResult(const CCDDBQueryResult& result);

//Methods
  CCDDBQueryResult& operator=(const CCDDBQueryResult& result);

//Data variables
  CString m_sCategory;  //The category of this CD
  DWORD   m_dwDiscID;   //The CDDB DISCID of this CD
  CString m_sArtist;    //The Artist of this CD
  CString m_sTitle;     //The Disc title of the CD

protected:
};

//Class which represents a single instance of the
//results as returned from CCDDB::Read and used in 
//the call to CCDDB::Submit
class MFCCDDB_EXT_CLASS CCDDBRecord
{
public:
  CCDDBRecord();
  CCDDBRecord(const CCDDBRecord& record);

//Methods
  CCDDBRecord& operator=(const CCDDBRecord& result);
  
//Data variables
  CArray<int, int&> m_TrackOffsets;             //The offsets of each track in frames
  int               m_nDiskLength;              //The Disc length in seconds
  int               m_nDatabaseRevision;        //The database revision number of this record
  CString           m_sClientName;              //The name of the client which has submitted this record
  CString           m_sClientVersion;           //The version of the client which has submitted this record
  CString           m_sClientComments;          //Optional client comments
  DWORD             m_dwDiscID;                 //The CDDB DISC-ID of this CD
  CString           m_sArtist;                  //The Artist of this CD
  CString           m_sTitle;                   //The Disc title of the CD
  CStringArray      m_TrackTitles;              //All the track titles on this CD
  CString           m_ExtendedData;             //The extended data for this CD
  CStringArray      m_ExtendedTrackData;        //The extended data for each track on this CD
  CArray<int, int&> m_PlayOrder;                //The track play order if any for this CD
  int               m_nYear;                    //Year in which the CD was released
  CString           m_sGenre;                   //The genre of the disc in textual form
};

//Class which represents the result as returned from
//CCDDB::Status
class MFCCDDB_EXT_CLASS CCDDBStatus
{
public:
//Constructors / Destructors
  CCDDBStatus();

//Memeber variables
  int               m_nCurrentProtocol; //The servers current operating protocol level
  int               m_nMaxProtocol;     //The maximum supported protocol level
  BOOL              m_bGetsAllowed;     //Whether or not the client is allowed to get log information
  BOOL              m_bUpdatesAllowed;  //Whether or not the client is allowed to initiate a database update
  BOOL              m_bPostingAllowed;  //Whether or not the client is allowed to post new entries
  BOOL              m_bQuotes;          //Whether or not quoted arguements are enabled
  int               m_nCurrentUsers;    //The number of users currently connected to the server
  int               m_nMaxUsers;        //The number of users that can concurrently connect to the server
  BOOL              m_bStripExtended;   //Whether or not extended data is stripped by the server before presented to the user
  int               m_nDatabaseEntries; //Total number of entries in the database
  CStringArray      m_Categories;       //The categories held in the database
  CArray<int, int&> m_CategoryEntries;  //The number of entries for each category in the database
  CStringArray      m_PendingSites;     //List of sites that are fed new database entries from this server
  CArray<int, int&> m_PendingEntries;   //The number of entries that have yet to be transmitted to each site
};

//Simple class to store a tracks position in MSF format. This is
//required for computing the CDDB DISCID
class MFCCDDB_EXT_CLASS CCDDBTrackPosition
{
public:
//Constructors / Destructors
  CCDDBTrackPosition();
  CCDDBTrackPosition(const CCDDBTrackPosition& position);

//Methods
  CCDDBTrackPosition& operator=(const CCDDBTrackPosition& position);

//Member variables
  int m_nMinute;
  int m_nSecond;
  int m_nFrame;
};

//The main CDDB support class
class MFCCDDB_EXT_CLASS CCDDB
{
public:
//Constructors / Destructors
  CCDDB();

//Local Methods
  void    GetCDROMDrives(CStringArray& drives);
  DWORD   ComputeDiscIDUsingMCI(LPCTSTR pszDrive = NULL);
  DWORD   ComputeDiscIDUsingDeviceIoControl(LPCTSTR pszDrive = NULL);
  DWORD   ComputeDiscID(const CArray<CCDDBTrackPosition, CCDDBTrackPosition&>& tracks);
  void    GetTrackPositionsUsingMCI(CArray<CCDDBTrackPosition, CCDDBTrackPosition&>& tracks, LPCTSTR pszDrive = NULL);
  void    GetTrackPositionsUsingDeviceIoControl(CArray<CCDDBTrackPosition, CCDDBTrackPosition&>& tracks, LPCTSTR pszDrive);
  
//Remote CDDB access methods
  void    SetCDDBProtocolVersion(int nProtocolVersion);
  int     GetCDDBProtocolVersion() const { return m_nProtocolVersion; };
  void    Sites(CArray<CCDDBSite, CCDDBSite&>& sites, const CString& sServer = _T("freedb.freedb.org"), const CString& sAddress = _T("/~cddb/cddb.cgi"), int nPort = 80);
  void    Sites(CArray<CCDDBSite, CCDDBSite&>& sites, const CCDDBSite& server);
  void    Categories(const CCDDBSite& server, CStringArray& categories);
  void    Status(const CCDDBSite& server, CCDDBStatus& status);
  void    Query(const CCDDBSite& server, DWORD dwDiscID, const CArray<CCDDBTrackPosition, CCDDBTrackPosition&>& tracks, 
                CArray<CCDDBQueryResult, CCDDBQueryResult&>& results); 
  void    Read(const CCDDBSite& server, DWORD dwDiscID, const CString& sCategory, CCDDBRecord& record);
  BOOL    Read(const CString& sDirectory, DWORD dwDiscID, BOOL bStandardForm, CCDDBRecord& record, CString& sFileSystemGenre);
  void    Submit(const CCDDBSite& server, const CString& sCategory, const CString& sEmailAddress, 
                 CCDDBRecord& record, const CString& sRejectionNotice, BOOL bReallySubmit = TRUE);
  void    MessageOfTheDay(CString& sMessage, const CString& sServer = _T("freedb.freedb.org"), const CString& sAddress = _T("/~cddb/cddb.cgi"), int nPort = 80);
  void    MessageOfTheDay(const CCDDBSite& server, CString& sMessage);

//Simple Accessors / Mutators
  void    SetTimeout(DWORD dwTimeout) { m_dwTimeout = dwTimeout; };
  DWORD   GetTimeout() const { return m_dwTimeout; };
  CString GetLastCommandResponse() const { return m_sLastCommandResponse; };
  CString GetProductName() const { return m_sProductName; };
  void    SetProductName(const CString& sProductName) { m_sProductName = sProductName; };
  CString GetProductVersion() const { return m_sProductVersion; };
  void    SetProductVersion(const CString& sProductVersion) { m_sProductVersion = sProductVersion; };
  CString GetHostName() { return m_sHostName; };
  void    SetHostname(const CString& sHostname);
  CString GetUserName() { return m_sUserName; };
  void    SetUserName(const CString& sUserName);
  
//Proxy support
  void    SetProxyDetails(const CString& sHost, int nPort, const CString& sUser, const CString& sPassword);
  void    SetProxyHost(const CString& sProxyHost) { m_sProxyHost = sProxyHost; };
  void    SetProxyPort(int nPort) { m_nProxyPort = nPort; };
  void    SetProxyUser(const CString& sUser) { m_sProxyUser = sUser; };
  void    SetProxyPassword(const CString& sPassword) { m_sProxyPassword = sPassword; };
  CString GetProxyHost() const { return m_sProxyHost; };
  int     GetProxyPort() const { return m_nProxyPort; };
  CString GetProxyUser() const { return m_sProxyUser; };
  CString GetProxyPassword() const { return m_sProxyPassword; };

//Internal virtual functions which derived classes can customize if required
  virtual CString  GetHelloCommand();
  virtual BOOL     ReadResponse(CWSocket& socket, LPSTR pszTerminator, CStringA& sResponse, DWORD dwHint = 0);
  virtual void     PrepareTextForSubmit(CString& sText);
  virtual void     StripNonAlphaOrDigits(CString& sValue);
  virtual LPCSTR   FindHTTPBody(LPCSTR pszResponse);
  virtual int      GetCDDBReponseCode(LPCSTR pszBody);
  virtual BOOL     SplitSemiColonLine(const CStringA& sLine, CStringA& sField, CStringA& sValue);
  virtual BOOL     ParseSitesBody(LPCSTR pszBody, CArray<CCDDBSite, CCDDBSite&>& sites);
  virtual BOOL     ParseSiteBodyHelper(CCDDBSite& site, const CStringA& sLine);
  virtual BOOL     ParseCategoriesBody(LPCSTR pszBody, CStringArray& categories);
  virtual BOOL     ParseStatusBody(LPCSTR pszBody, CCDDBStatus& status);
  virtual BOOL     ParseQueryBody(LPCSTR pszBody, const CString& sDiscID, CArray<CCDDBQueryResult, CCDDBQueryResult&>& results);
  virtual BOOL     ParseQueryBodyHelper(CCDDBQueryResult& result, const CStringA& sLine);
  virtual BOOL     ParseRecordBody(LPCSTR pszBody, CCDDBRecord& record);
  virtual BOOL     ParseRecordBodyHelper(CCDDBRecord& record, const CStringA& sRecord);
  virtual void     ParseRecordBodyFromFile(DWORD dwDiscID, const CString& sFile, BOOL bStandardForm, CCDDBRecord& record);
  virtual BOOL     ParseMessageBody(LPCSTR pszBody, CString& sMessage);
  virtual BOOL     ParseSubmitBody(LPCSTR pszBody);
  virtual CString  GetProxyHeader(const CString& sHost);
  virtual CString  FormSubmitLine(const CString& sKeyword, const CString& sValue);
  virtual CString  MapFromProtocolString(const CStringA& sInput);
  virtual CStringA MapToProtocolString(const CString& sInput);
  virtual void     GetNextLine(LPCSTR& pszLine, CStringA& sLine);
  virtual BOOL     ParseCommentLine(const CStringA& sLine, CStringA& sValue);
  virtual BOOL     ParseKeywordLine(const CStringA& sLine, CStringA& sKeyword, CStringA& sValue);

//Static methods
  static void ThrowCDDBException(DWORD dwError = 0, DWORD Facility = FACILITY_WIN32, const CString& sLastResponse = _T(""));
  static void ThrowCDDBException(HRESULT hr, const CString& sLastResponse = _T(""));
  static void ThrowCDDBMMException(DWORD dwError, const CString& sLastResponse = _T(""));

protected:
//Enums
  enum
  {
    ReadResponseQuery = 1,
    ReadResponseSubmit = 2
  };

//Member variables
  CString m_sLastCommandResponse;
  DWORD   m_dwTimeout;
  CString m_sProductName;
  CString m_sProductVersion;
  int     m_nProtocolVersion;
  CString m_sProxyHost;
  int     m_nProxyPort;
  CString m_sProxyUser;
  CString m_sProxyPassword;
  CString m_sHostName;
  CString m_sUserName;
};

#endif //__MFCCDDB_H__
