#include "stdafx.h"
#include "app.h"
#include "mfccddb.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

//The one and only CWinApp instance
CWinApp theApp;

void _tmain()
{
 	//initialize MFC and print and error on failure (Note we must put up with some /analyze warnings for AfxWinInit even in VC 2008)
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	  return;
  
  //Initialise the Winsock stack
  if (!AfxSocketInit())
    return;

  try
  {
    //Create the CDDB instance
    CCDDB cddb;  

    //Uncomment the following block of code to test the record parsing code
    //without having to connect to the internet. 
    /*
    CFile file;
    VERIFY(file.Open(_T("cddbrecord.txt"), CFile::modeRead | CFile::shareDenyWrite));
    ULONGLONG dwLength = file.GetLength();
    VERIFY(dwLength < 1024*1024); //Limit our memory allocation to 1 MB for testing purposes
    LPSTR pData = new char[static_cast<unsigned int>(dwLength+1)];
    file.Read(pData, static_cast<UINT>(dwLength));
    pData[dwLength] = '\0'; //Don't forget to NULL terminate
    CCDDBRecord recordtoRead;
    cddb.ParseRecordBodyHelper(recordtoRead, pData);
    delete [] pData;
    file.Close();

    //Display all the album details
    _tprintf(_T("\nRetrieved the following album details\n"));
    _tprintf(_T("Album Title: %s\n"), recordtoRead.m_sTitle.operator LPCTSTR());
    _tprintf(_T("Album Artist: %s\n"), recordtoRead.m_sArtist.operator LPCTSTR());
    _tprintf(_T("Disc Length (seconds): %d\n"), recordtoRead.m_nDiskLength);
    _tprintf(_T("Submitted via: %s %s %s\n"), recordtoRead.m_sClientName.operator LPCTSTR(), recordtoRead.m_sClientVersion.operator LPCTSTR(), recordtoRead.m_sClientComments.operator LPCTSTR());
    _tprintf(_T("Database Revision: %d\n"), recordtoRead.m_nDatabaseRevision);
    _tprintf(_T("DISCID: %08x\n"), recordtoRead.m_dwDiscID);

    //Display the track names 
    _tprintf(_T("\nTrack Names are:\n"));
    for (int i=0; i<recordtoRead.m_TrackTitles.GetSize(); i++)
      _tprintf(_T("Track %d: %s\n"), i+1, recordtoRead.m_TrackTitles.ElementAt(i).operator LPCTSTR());

    //Display the Extended track data 
    _tprintf(_T("\nExtended Track Data:\n"));
    for (int i=0; i<recordtoRead.m_ExtendedTrackData.GetSize(); i++)
      _tprintf(_T("Track %d: %s\n"), i+1, recordtoRead.m_ExtendedTrackData.ElementAt(i).operator LPCTSTR());

    //Display the Extended data 
    _tprintf(_T("\nExtended Data: %s\n"), recordtoRead.m_ExtendedData.operator LPCTSTR());

    //Display the track offsets 
    _tprintf(_T("\nTrack Offsets:\n"));
    for (int i=0; i<recordtoRead.m_TrackOffsets.GetSize(); i++)
      _tprintf(_T("%d\n"), recordtoRead.m_TrackOffsets.ElementAt(i));

    //Display the Playorder
    _tprintf(_T("Playorder :\n"));
    for (int i=0; i<recordtoRead.m_PlayOrder.GetSize(); i++)
      _tprintf(_T("%d\n"), recordtoRead.m_PlayOrder.ElementAt(i));

    //Try to do a submit with recordtoRead
    CCDDBSite submitsite(FALSE);
    submitsite.m_sSite = _T("freedb.freedb.org");
    cddb.Submit(submitsite, recordtoRead.m_sGenre, _T("pjna@naughter.com"), recordtoRead, _T("Test submission by MfcCDDB code"), FALSE);
    */

    //Test out getting all the CDROM drive letters
    CStringArray drives;
    cddb.GetCDROMDrives(drives);

    CString sMsg1;
    sMsg1 = _T("CDROM Drives on this machine are: ");
    for (INT_PTR i=0; i<drives.GetSize(); i++)
    {
      sMsg1 += drives.ElementAt(i);
      if (i != drives.GetUpperBound())
        sMsg1 += _T(',');
    }
    _tprintf(_T("%s\n"), sMsg1.operator LPCTSTR());
    if (drives.GetSize() == 0)
    {
      _tprintf(_T("There are no CD drives on this computer\n"));
      return;
    }

    //Test out the DISCID function
    DWORD dwDiscID;
    _tprintf(_T("Please insert an Audio CD into the first CD drive>\n"));
    int nCharRead = _getch();
    nCharRead; //To get rid of level 4 warning
    try
    {
      dwDiscID = cddb.ComputeDiscIDUsingMCI(drives.GetAt(0));
      _tprintf(_T("The CDDB DISCID (using MCI) for this Audio CD is %08x\n"), dwDiscID);
      dwDiscID = cddb.ComputeDiscIDUsingDeviceIoControl(drives.GetAt(0));
      _tprintf(_T("The CDDB DISCID (using DeviceIoControl) for this Audio CD is %08x\n"), dwDiscID);
    }
    catch(CCDDBException* pEx)
    { 
      CString sMsg;
      _tprintf(_T("An error occurred while retrieving the DISC ID, Error:%08X, Text:%s\n"), pEx->m_hr, pEx->GetErrorMessage().operator LPCTSTR());
      pEx->Delete();
    }

    //Uncomment this line and change the details if you want to try out the MfcCDDB support for proxies and proxy authentication
    //cddb.SetProxyDetails(_T("your_proxys_address"), 8080, _T("a_user"), _T("some_password")); //use if you require authenticated proxy connections
    //cddb.SetProxyDetails(_T("your_proxys_address"), 8080, _T(""), _T(""));                    //use if you require a proxy without authentication

    //Test out the Sites command
    CArray<CCDDBSite, CCDDBSite&> sites;
    cddb.Sites(sites);

    _tprintf(_T("Retrieved the following CDDB sites from the CDDB server\n"));
    CCDDBSite site2;
    BOOL bFoundFirstPort80 = FALSE;
    for (INT_PTR i=0; i<sites.GetSize(); i++)
    {
      CCDDBSite s = sites.ElementAt(i);
      CString sLat;
      sLat.Format(_T("%c%03d.%02d"), s.m_bNorthing ? _T('N') : _T('S'), s.m_nLatitudeMinutes / 60, s.m_nLatitudeMinutes % 60);
      CString sLong;
      sLong.Format(_T("%c%03d.%02d"), s.m_bEasting ? _T('E') : _T('W'), s.m_nLongitudeMinutes / 60, s.m_nLongitudeMinutes % 60);
      _tprintf(_T("%s, %d, %s, %s, %s, %s\n"), s.m_sSite.operator LPCTSTR(), s.m_nPort, s.m_sAddress.operator LPCTSTR(), sLat.operator LPCTSTR(), sLong.operator LPCTSTR(), s.m_sDescription.operator LPCTSTR());
      
      //Hive away the first site which works on port 80
      if (!bFoundFirstPort80 && s.m_nPort == 80)
      {
        bFoundFirstPort80 = TRUE;
        site2 = s;
      }
    }

    if (!sites.GetSize())
      return;
    if (!bFoundFirstPort80)
      return;

    //Try the other version of the site function
    cddb.Sites(sites, site2);

    _tprintf(_T("Retrieved the following CDDB sites from %s\n"), site2.m_sSite.operator LPCTSTR());
    for (INT_PTR i=0; i<sites.GetSize(); i++)
    {
      CCDDBSite s = sites.ElementAt(i);
      CString sLat;
      sLat.Format(_T("%c%03d.%02d"), s.m_bNorthing ? _T('N') : _T('S'), s.m_nLatitudeMinutes / 60, s.m_nLatitudeMinutes % 60);
      CString sLong;
      sLong.Format(_T("%c%03d.%02d"), s.m_bEasting ? _T('E') : _T('W'), s.m_nLongitudeMinutes / 60, s.m_nLongitudeMinutes % 60);
      _tprintf(_T("%s, %d, %s, %s, %s, %s\n"), s.m_sSite.operator LPCTSTR(), s.m_nPort, s.m_sAddress.operator LPCTSTR(), sLat.operator LPCTSTR(), sLong.operator LPCTSTR(), s.m_sDescription.operator LPCTSTR());
    }

    //Try out the message of the day function
    CString sMessage;
    cddb.MessageOfTheDay(site2, sMessage);

    //Try the Categories function
    CStringArray sCategories;
    CCDDBSite site5;
    site5.m_sSite = _T("freedb.freedb.org");
    cddb.Categories(site5, sCategories);
    _tprintf(_T("Retrieved the following CDDB categories from %s\n"), site5.m_sSite.operator LPCTSTR());
    for (INT_PTR i=0; i<sCategories.GetSize(); i++)
      _tprintf(_T("%s\n"), sCategories.ElementAt(i).operator LPCTSTR());

    //Try out the Status function
    CCDDBSite site3(TRUE);
    site3.m_sSite = _T("freedb.freedb.org");
    CCDDBStatus status;
    cddb.Status(site3, status);
    //Display the results
    _tprintf(_T("\nStatus of the site %s is as follows\n"), site3.m_sSite.operator LPCTSTR());
    _tprintf(_T("Current protocol: %d\n"), status.m_nCurrentProtocol);
    _tprintf(_T("Max protocol: %d\n"), status.m_nMaxProtocol);
    _tprintf(_T("Gets allowed: %d\n"), status.m_bGetsAllowed);
    _tprintf(_T("Updates allowed: %d\n"), status.m_bUpdatesAllowed);
    _tprintf(_T("Posting: %d\n"), status.m_bPostingAllowed);
    _tprintf(_T("Quotes: %d\n"), status.m_bQuotes);
    _tprintf(_T("Current Users: %d\n"), status.m_nCurrentUsers);
    _tprintf(_T("Max Users: %d\n"), status.m_nMaxUsers);
    _tprintf(_T("Strip Extended: %d\n"), status.m_bStripExtended);
    _tprintf(_T("Database Entries: %d\n"), status.m_nDatabaseEntries);
    for (INT_PTR i=0; i<status.m_Categories.GetSize(); i++)
      _tprintf(_T("%s has %d entries\n"), status.m_Categories.ElementAt(i).operator LPCTSTR(), status.m_CategoryEntries.ElementAt(i));
    for (INT_PTR i=0; i<status.m_PendingSites.GetSize(); i++)
      _tprintf(_T("%s has %d pending entries\n"), status.m_PendingSites.ElementAt(i).operator LPCTSTR(), status.m_PendingEntries.ElementAt(i));

    //Test out the GetTrackPositionsUsingMCI function
    CArray<CCDDBTrackPosition, CCDDBTrackPosition&> tracks;
    cddb.GetTrackPositionsUsingMCI(tracks);
    
    //Test out the GetTrackPositionsUsingDeviceIOControl function
    cddb.GetTrackPositionsUsingDeviceIoControl(tracks, drives.GetAt(0));

    //Test out the ComputeDiscID function
    dwDiscID = cddb.ComputeDiscIDUsingMCI();

    //Test out the query function
    CArray<CCDDBQueryResult, CCDDBQueryResult&> results;
    cddb.Query(site3, dwDiscID, tracks, results);

    _tprintf(_T("Retrieved the following matches for the inserted audio CD\n"));
    for (INT_PTR i=0; i<results.GetSize(); i++)
    {
      CCDDBQueryResult& result = results.ElementAt(i);
      _tprintf(_T("%08x, %s / %s\n"), result.m_dwDiscID, result.m_sArtist.operator LPCTSTR(), result.m_sTitle.operator LPCTSTR());
    }

    //Try out the read function
    CCDDBQueryResult& result = results.ElementAt(0);
    CCDDBRecord record;
    cddb.Read(site3, result.m_dwDiscID, result.m_sCategory, record);

    //Display all the album details
    _tprintf(_T("\nRetrieved the following album details\n"));
    _tprintf(_T("Album Title: %s\n"), record.m_sTitle.operator LPCTSTR());
    _tprintf(_T("Album Artist: %s\n"), record.m_sArtist.operator LPCTSTR());
    _tprintf(_T("Disc Length (seconds): %d\n"), record.m_nDiskLength);
    _tprintf(_T("Submitted via: %s %s %s\n"), record.m_sClientName.operator LPCTSTR(), record.m_sClientVersion.operator LPCTSTR(), record.m_sClientComments.operator LPCTSTR());
    _tprintf(_T("Database Revision: %d\n"), record.m_nDatabaseRevision);
    _tprintf(_T("DISCID: %08x\n"), record.m_dwDiscID);

    //Display the track names 
    _tprintf(_T("\nTrack Names are:\n"));
    for (INT_PTR i=0; i<record.m_TrackTitles.GetSize(); i++)
      _tprintf(_T("Track %d: %s\n"), i+1, record.m_TrackTitles.ElementAt(i).operator LPCTSTR());

    //Display the Extended track data 
    _tprintf(_T("\nExtended Track Data:\n"));
    for (INT_PTR i=0; i<record.m_ExtendedTrackData.GetSize(); i++)
      _tprintf(_T("Track %d: %s\n"), i+1, record.m_ExtendedTrackData.ElementAt(i).operator LPCTSTR());

    //Display the Extended data 
    _tprintf(_T("\nExtended Data: %s\n"), record.m_ExtendedData.operator LPCTSTR());

    //Display the track offsets 
    _tprintf(_T("\nTrack Offsets:\n"));
    for (INT_PTR i=0; i<record.m_TrackOffsets.GetSize(); i++)
      _tprintf(_T("%d\n"), record.m_TrackOffsets.ElementAt(i));

    //Display the Playorder
    _tprintf(_T("Playorder :\n"));
    for (INT_PTR i=0; i<record.m_PlayOrder.GetSize(); i++)
      _tprintf(_T("%d\n"), record.m_PlayOrder.ElementAt(i));

    //Try out the Submit function using the test mode
    VERIFY(FALSE); //please change the email address below to your own to send
                   //any rejection emails to you rather than the made up address. 
    CCDDBSite site6(FALSE);
    site6.m_sSite = _T("freedb.freedb.org");
    cddb.Submit(site6, result.m_sCategory, _T("someone@somedomain.com"), record, _T("Test submission by MfcCDDB code"), FALSE);


    /* 
    //Uncomment the following to test out local read support, please change the first parameter
    //to suit where you have downloaded the freedb database to. The second contains the discid 
    //to lookup, the third parameter is for if you are using the standard or alternate form of 
    //the database, the fourth parameter contains the record details if the read was successful
    //and the final parameter contains the genre of the record as read from the database
    CString sFileSystemGenre;
    CCDDBRecord record2;
    //BOOL bSuccess2 = cddb.Read(_T("c:\\multimedia\\freedb database"), 0x0a0c4c12, FALSE, record2, sFileSystemGenre); //to use the alternative (i.e. not Hard links) DB format
    BOOL bSuccess2 = cddb.Read(_T("c:\\multimedia\\freedb database"), 0x0a0c4c12, TRUE, record2, sFileSystemGenre); //To use the standard DB format
    if (!bSuccess2)
      return FALSE;

    //Display all the album details
    _tprintf(_T("\nRetrieved the following album details from local disk\n"));
    _tprintf(_T("File System Genre: %s\n"), sFileSystemGenre.operator LPCTSTR());
    _tprintf(_T("Album Title: %s\n"), record2.m_sTitle.operator LPCTSTR());
    _tprintf(_T("Album Artist: %s\n"), record2.m_sArtist.operator LPCTSTR());
    _tprintf(_T("Disc Length (seconds): %d\n"), record2.m_nDiskLength);
    _tprintf(_T("Submitted via: %s %s %s\n"), record2.m_sClientName.operator LPCTSTR(), record2.m_sClientVersion.operator LPCTSTR(), record2.m_sClientComments.operator LPCTSTR());
    _tprintf(_T("Database Revision: %d\n"), record2.m_nDatabaseRevision);
    _tprintf(_T("DISCID: %08x\n"), record2.m_dwDiscID);

    //Display the track names 
    _tprintf(_T("\nTrack Names are:\n"));
    for (INT_PTR i=0; i<record2.m_TrackTitles.GetSize(); i++)
      _tprintf(_T("Track %d: %s\n"), i+1, record2.m_TrackTitles.ElementAt(i).operator LPCTSTR());

    //Display the Extended track data 
    _tprintf(_T("\nExtended Track Data:\n"));
    for (INT_PTR i=0; i<record.m_ExtendedTrackData.GetSize(); i++)
      _tprintf(_T("Track %d: %s\n"), i+1, record2.m_ExtendedTrackData.ElementAt(i).operator LPCTSTR());

    //Display the Extended data 
    _tprintf(_T("\nExtended Data: %s\n"), record2.m_ExtendedData.operator LPCTSTR());

    //Display the track offsets 
    _tprintf(_T("\nTrack Offsets:\n"));
    for (INT_PTR i=0; i<record2.m_TrackOffsets.GetSize(); i++)
      _tprintf(_T("%d\n"), record2.m_TrackOffsets.ElementAt(i));

    //Display the Playorder
    _tprintf(_T("Playorder :\n"));
    for (INT_PTR i=0; i<record2.m_PlayOrder.GetSize(); i++)
      _tprintf(_T("%d\n"), record2.m_PlayOrder.ElementAt(i));
    */
  }
  catch(CCDDBException* pEx)
  {
    CString sMsg;
    _tprintf(_T("An unexpected CCDDBException has occurred, Error:%X, Text:%s\n"), pEx->m_hr, pEx->GetErrorMessage().operator LPCTSTR());
    pEx->Delete();
  }
}
